# CollectKitFiles.mak
#       RBLandau 20090409
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# Collect all the files for the printer prototype proxy provider 
# into a clean directory structure, so it can be zipped up for 
# distribution.  
# This doesn't use any real makefile rules or dependency 
# evaluation; it's just a list of the files to be packed up 
# so we don't miss anything.  
#

# Where are the source and destination directory trees
# relative to the current directory?
SRC = .
DSTTOP = ../kit
DST = ../kit/provider

# a l l 
# Default target
all :   makedirs \
        deversionfiles \
        pythonfiles shellfiles \
        datfiles inifiles \
        docfiles logfiles \
        mibfiles moffiles \
        otherfiles \
        zipfiles \
        donedone

# c l e a n 
# Remove all the files, but leave the directory structure.
clean : 
	rm -f $(DSTTOP)/*.*
	rm -f $(DST)/*.*
	rm -f $(DST)/dat/*.*
	rm -f $(DST)/ini/*.*
	rm -f $(DST)/docs/*.*
	rm -f $(DST)/mibs/*.*
	rm -f $(DST)/mofs/*.*

# m a k e d i r s 
# At the destination, create any directories that aren't already there.  
makedirs : $(DSTTOP) $(DST) $(DST)/dat $(DST)/ini $(DST)/docs $(DST)/mibs $(DST)/mofs
	# target makedirs
$(DSTTOP) :
	if [ ! -d $(DSTTOP) ] ; then mkdir $(DSTTOP) ; fi
$(DST) :
	if [ ! -d $(DST) ] ; then mkdir $(DST) ; fi
$(DST)/dat :
	if [ ! -d $(DST)/dat ] ; then mkdir $(DST)/dat ; fi
$(DST)/ini :
	if [ ! -d $(DST)/ini ] ; then mkdir $(DST)/ini ; fi
$(DST)/docs :
	if [ ! -d $(DST)/docs ] ; then mkdir $(DST)/docs ; fi
$(DST)/mibs :
	if [ ! -d $(DST)/mibs ] ; then mkdir $(DST)/mibs ; fi
$(DST)/mofs :
	if [ ! -d $(DST)/mofs ] ; then mkdir $(DST)/mofs ; fi

# l i s t a l l 
# Show me all the files.  
listall : 
	ls -R $(DST)

# l i s t a l l l o n g 
# Show me how big the files are, too.
listalllong : 
	ls -lR $(DST)

# d e v e r s i o n f i l e s 
# Remove the version numbers from some of the files.
# This has to be done carefully to provide a version-neutral
# name for the rest of the makefile to use. 
deversionfiles : 
	# target = deversionfiles
	if [ -e $(SRC)/CollectKitFiles.mak ] ; then rm -f $(SRC)/CollectKitFiles.mak ; fi 
	cp `ls $(SRC)/CollectKitFiles*.mak | sort | grep -v '.bak' | tail -1` $(SRC)/CollectKitFiles.mak

	if [ -e $(SRC)/docs/PPPP_ClassesAndProcesses.doc ] ; then rm -f $(SRC)/docs/PPPP_ClassesAndProcesses.doc ; fi 
	cp `ls $(SRC)/docs/PPPP_ClassesAndProcesses*.doc | sort | tail -1` $(SRC)/docs/PPPP_ClassesAndProcesses.doc

	if [ -e $(SRC)/docs/PPPP_HowToReadOutputFiles.doc ] ; then rm -f $(SRC)/docs/PPPP_HowToReadOutputFiles.doc ; fi 
	cp `ls $(SRC)/docs/PPPP_HowToReadOutputFiles*.doc | sort | tail -1` $(SRC)/docs/PPPP_HowToReadOutputFiles.doc

	if [ -e $(SRC)/docs/PPPP_HowToRun.doc ] ; then rm -f $(SRC)/docs/PPPP_HowToRun.doc ; fi 
	cp `ls $(SRC)/docs/PPPP_HowToRun*.doc | sort | tail -1` $(SRC)/docs/PPPP_HowToRun.doc

	if [ -e $(SRC)/docs/PPPP_WhatsInTheDistribution.doc ] ; then rm -f $(SRC)/docs/PPPP_WhatsInTheDistribution.doc ; fi 
	cp `ls $(SRC)/docs/PPPP_WhatsInTheDistribution*.doc | sort | tail -1` $(SRC)/docs/PPPP_WhatsInTheDistribution.doc

	if [ -e $(SRC)/docs/Visio-Printer.pdf ] ; then rm -f $(SRC)/docs/Visio-Printer.pdf ; fi 
	cp `ls $(SRC)/docs/Visio-Printer*.pdf | sort | tail -1` $(SRC)/docs/Visio-Printer.pdf

# p y t h o n f i l e s 
# Get the Python source files, one by one.  
pythonfiles : 
# Provider runtime files.
	# target = pythonfiles 
	cp $(SRC)/assoc.py $(DST)/
	cp $(SRC)/calcsetup.py $(DST)/
	cp $(SRC)/calcvalues.py $(DST)/
	cp $(SRC)/cimclass.py $(DST)/
	cp $(SRC)/cimprop.py $(DST)/
	cp $(SRC)/datastore.py $(DST)/
	cp $(SRC)/dicts.py $(DST)/
	cp $(SRC)/NewTrace.py $(DST)/
	cp $(SRC)/printer.py $(DST)/
	cp $(SRC)/provini.py $(DST)/
	cp $(SRC)/request.py $(DST)/
	cp $(SRC)/snmpcli.py $(DST)/
	cp $(SRC)/subsystem.py $(DST)/
	# Unit test files.  
	cp $(SRC)/prov-test-enum-all.py $(DST)/
	cp $(SRC)/prov-test-enum-assoc.py $(DST)/
	cp $(SRC)/prov-test-enum-one.py $(DST)/

# s h e l l f i l e s 
# Get the shell scripts that direct tests.  
shellfiles:
	# target = shellfiles
	cp $(SRC)/prov-test-allfiles.sh $(DST)/
	cp $(SRC)/prov-test-analyze-log.sh $(DST)/
	cp $(SRC)/prov-test-enum-all.sh $(DST)/
	cp $(SRC)/prov-test-enum-assoc.sh $(DST)/
	cp $(SRC)/prov-test-enum-one.sh $(DST)/

# o t h e r f i l e s 
# Any other files that we need to pick up.
otherfiles : 
	# target = otherfiles
	cp $(SRC)/CollectKitFiles.mak $(DST)/

# d a t f i l e s 
# Get the SNMP dump files for the set of test printers.  
datfiles : 
	# target = datfiles
	cp $(SRC)/dat/delltest2.ini $(DST)/dat/
	cp $(SRC)/dat/delltest3.ini $(DST)/dat/
	cp $(SRC)/dat/delltest4-bw5300.ini $(DST)/dat/
	cp $(SRC)/dat/hptest1-old.ini $(DST)/dat/
	cp $(SRC)/dat/hptest2-bw.ini $(DST)/dat/
	cp $(SRC)/dat/hptest3-color.ini $(DST)/dat/
	cp $(SRC)/dat/hptest4-colorij.ini $(DST)/dat/
	cp $(SRC)/dat/konicatest1-color.ini $(DST)/dat/
	cp $(SRC)/dat/konicatest2-color2350.ini $(DST)/dat/
	cp $(SRC)/dat/konicatest3-color2550.ini $(DST)/dat/
	cp $(SRC)/dat/lextest1-color.ini $(DST)/dat/
	cp $(SRC)/dat/okitest1-color.ini $(DST)/dat/
	cp $(SRC)/dat/sharptest1-color.ini $(DST)/dat/
	cp $(SRC)/dat/snmpvalues.ini $(DST)/dat/
	cp $(SRC)/dat/xeroxtest1-color.ini $(DST)/dat/
	cp $(SRC)/dat/xeroxtest3-wc7665.ini $(DST)/dat/
	cp $(SRC)/dat/xeroxtest4-wc4260.ini $(DST)/dat/
	cp $(SRC)/dat/xeroxtest5-wc5050.ini $(DST)/dat/
	cp $(SRC)/dat/xeroxtest6-wc7345.ini $(DST)/dat/

# i n i f i l e s 
# Get the initialization files that describe printers, properties, etc.
inifiles : 
	# target = inifiles
	cp $(SRC)/ini/algs.ini $(DST)/ini/
	cp $(SRC)/ini/assoc.ini $(DST)/ini/
	cp $(SRC)/ini/bins.ini $(DST)/ini/
	cp $(SRC)/ini/classprops.ini $(DST)/ini/
	cp $(SRC)/ini/mibs.ini $(DST)/ini/
	cp $(SRC)/ini/papersizes.ini $(DST)/ini/
	cp $(SRC)/ini/printers.ini $(DST)/ini/
	cp $(SRC)/ini/propenums.ini $(DST)/ini/
	cp $(SRC)/ini/tables.ini $(DST)/ini/
	cp $(SRC)/ini/varsubs.ini $(DST)/ini/

# d o c f i l e s 
# Get the separate documentation files. 
docfiles : 
	# target = docfiles
	cp $(SRC)/docs/PPPP_ClassesAndProcesses.doc         $(DST)/docs/
	cp $(SRC)/docs/PPPP_HowToReadOutputFiles.doc        $(DST)/docs/
	cp $(SRC)/docs/PPPP_HowToRun.doc                    $(DST)/docs/
	cp $(SRC)/docs/PPPP_WhatsInTheDistribution.doc      $(DST)/docs/
	cp $(SRC)/docs/Visio-Printer.pdf                    $(DST)/docs/

# l o g f i l e s 
# Get the latest combined log file and its analyzed log file.  
logfiles : 
	# target = logfiles
	if [ -n "`ls $(SRC)/prov-test-allfiles-2*.log`" ] ; \
    then \
        cp `ls $(SRC)/prov-test-allfiles-2*.log | sort | grep -v '.bak' | tail -1` $(DST)/ ; \
    fi 
	if [ -n "`ls $(SRC)/prov-test-allfiles-analyzed-2*.log`" ] ; \
    then \
        cp `ls $(SRC)/prov-test-allfiles-analyzed-2*.log | sort | grep -v '.bak' | tail -1` $(DST)/ ; \
    fi 

# m i b f i l e s 
# RFCs for the MIBs that are used.
# The foolishness around name manipulation here is to survive names 
# containing embedded blanks, if any.
mibfiles : 
	# target = mibfiles
	for fff in `ls $(SRC)/mibs/rfc*.txt | sed 's/ /____/g'`; do ggg=`echo $$fff | sed 's/____/ /g'`; echo Copying file "$$ggg"; cp "$$ggg" $(DST)/mibs/; done

# m o f f i l e s 
# DMTF MOF definitions for the printer device classes and their parents.  
# The foolishness around name manipulation here is to survive names 
# containing embedded blanks, if any.
moffiles : 
	# target = moffiles
	for fff in `ls $(SRC)/mofs/*.mof | sed 's/ /____/g'`; do ggg=`echo $$fff | sed 's/____/ /g'`; echo Copying file "$$ggg"; cp "$$ggg" $(DST)/mofs/; done

# z i p f i l e s 
# Package the kit area into a zip archive
zipfiles : 
	# target = zipfiles
	(cd $(DSTTOP); zip -R providerkit '*.*' -x '*.zip';)

# donedone
# Tell the user that we got to the end successfully, we think.  
donedone : 
	# target = donedone
	#
	# Completed!
	#

#end
