#!/usr/bin/python
# assoc.py
#                   RBLandau 20081215
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
#

from subsystem  import *
from cimclass   import *
from snmpcli    import cSnmpCli
from datastore  import *
from NewTrace   import TRC

# class  C A s s o c S u b s y s t e m
#
# Special class for associations

class CAssocSubsystem(CSubsystem):
    def __init__(self,myclassname):
        TRC.trace(1,"enter CAssocSubsystem init name|%s| id=|%s|" \
            %(myclassname,id(self)))
        CSubsystem.__init__(self,myclassname)
        self.sClassname = myclassname
        '''
        - To enumerate all instances of an association, 
            for all keys of dAssoc2classes
              if key starts with the desired classname
                get names of the two classes connected by the association
                for list of keys of class1
                  for list of keys of class2
                    store combined key in list of all keys of this assoc
                    how do we get the values of the key properties?  
                    create the instance of class1 if necessary (not for printer)
                    get InstanceID from that instance
            enumeration = this list of keys of class
        '''
        # Easier way: go thru the list of associations and their classes.
        for self.idxAssoc in range(0,len(lAssociations)):
            #self.stmpAssoc = lAssociations[self.idxAssoc]
            self.otmpAssocGroup = lAssocGroups[self.idxAssoc]
            self.stmpAssoc = self.otmpAssocGroup.sAssocname
            TRC.trace(4,"loop  assoc init idx|%s| class|%s|"\
                %(self.idxAssoc,self.stmpAssoc))
            self.lOtherClasses = lAssocClasses[self.idxAssoc]
            # If this is the right classname, then make the instances
            # of the class and record them in the group.  
            if self.stmpAssoc == self.sClassname:
                [self.sClass1,self.sClass2] = self.lOtherClasses
                TRC.trace(3,"proc  CAssocSubsystem found assoc|%s| other1|%s| other2|%s|"\
                    %(self.sClassname,self.sClass1,self.sClass2))
                self.lKeys = self.FindAllKeys(self.sClass1,self.sClass2)
                lAssocClassKeys.append([])
                lAssocClassKeys[self.idxAssoc] = self.lKeys
                TRC.trace(3,"proc  assoc|%s| classes|%s| allkeys|%s|"\
                    %(self.sClassname,self.lOtherClasses,self.lKeys))
                # For each keypair, make an instance object
                # and store it in a list of instances in the group.
                for self.lOneKeyPair in self.lKeys:
                    self.cAssocInst = CAssocInst(self.stmpAssoc,self.sClass1,self.sClass2)
                    self.cAssocInst.setKeyInfo(self.lOneKeyPair)
                    self.otmpAssocGroup.AddInstance(self.cAssocInst)
            else:
                lAssocClassKeys.append([])

    def FindAllKeys(self,myclass1,myclass2):
        TRC.trace(1,"enter CAssocSubsystem FindAllKeys classes|%s||%s|"\
                    % (myclass1,myclass2) )
        self.lInstances1 = datastore.cPrinter.getKeys(myclass1)
        self.lInstances2 = datastore.cPrinter.getKeys(myclass2)
        self.lCombos = list()
        for self.inst1 in self.lInstances1:
            for self.inst2 in self.lInstances2:
                if self.IsKeyValid(self.inst1,self.inst2):
                    self.lCombos.append([self.inst1,self.inst2])
        TRC.trace(3,"proc  CAssocSubsystem combos|%s|"%self.lCombos)        
        return self.lCombos
        
    def IsKeyValid(self,inst1,inst2):
        return True     # All keys are valid for the time being.  

    def StoreKey(self):
        pass

# Track instances of association classes.  

'''
class  C A s s o c I n s t 

For an association class, maintains the info needed to construct an instance:
- the classnames and RowId values of the two related class instances,
- the association class name
- (needs more?)
'''
class CAssocInst(object):
    def __init__(self,myAssocname,myClass1,myClass2):
        '''
        strings: three classnames, the assocname and the two classnames
        '''
        TRC.trace(1,"enter CAssocInst init assoc|%s| class1|%s| class2|%s| id|%s|"\
            %(myAssocname,myClass1,myClass2,id(self)))
        self.sAssocClassname = myAssocname
        self.lRelatedClassname = [myClass1,myClass2]
        self.lSNMPRowId = [0,0]
        
    def setKeyInfo(self,mylRowId):
        TRC.trace(1,"enter CAssocInst setinfo id|%s| relclass|%s| rowid=|%s|"\
            %(id(self),self.lRelatedClassname,mylRowId))
        self.lSNMPRowId = mylRowId
        
    def getInstInfo(self):
        return [
            self.lRelatedClassname,
            self.lSNMPRowId
            ]

'''
class  C A s s o c G r o u p 

For an association class, maintains 
- the names of the two related classes, 
- the names of the REF properties that relate those two classes, 
- a list of instances of that association (instances of CAssocInst).  
'''
class CAssocGroup(object):
    def __init__(self,myAssocname,mySeq,myClass1,myRef1,myClass2,myRef2):
        TRC.trace(1,"enter CAssocGroup init name|%s|seq|%s| class1|%s|ref|%s| class2|%s|ref|%s| id=|%s|"\
            %(myAssocname,myClass1,mySeq,myRef1,myClass2,myRef2,id(self)))
        self.sAssocname = myAssocname
        self.sClassname1 = myClass1
        self.sClassname2 = myClass2
        self.sRef1 = myRef1
        self.sRef2 = myRef2
        self.sSequence = mySeq
        self.ClearInstances()

    def ClearInstances(self):
        self.lInstances = list()

    # Add a CAssocInst instance to the list in this group.
    # Also inject a pointer to this group into the instance
    # for backwards navigation.  
    def AddInstance(self,myInstance):
        TRC.trace(1,"enter CAssocGroup AddInstance name|%s| inst=|%s|"\
            %(self.sAssocname,id(myInstance)))
        myInstance.oGroup = self
        myInstance.sSequence = self.sSequence
        self.lInstances.append(myInstance)

    def GetInstances(self):
        return self.lInstances

    def GetClassnames(self):
        return [self.sClassname1,self.sClassname2]

    def GetPropnames(self):
        return [self.sRef1,self.sRef2]


#############################################
# function  A s s o c F a c t o r y  
# Currently el cheapo elsif version.  
# Note: non-member function, called from outside to create instances.

def AssocFactory(myclassname,myInstObj):
    TRC.trace(1,"enter AssocClassFactory init name|%s|" \
        %(myclassname))
    inst = None
    if myclassname == "CIM_PrinterComponent":
        inst = CAssocPrinterComponent(myclassname,myInstObj)
    elif myclassname == "CIM_ConcreteComponent":
        inst = CAssocConcreteComponent(myclassname,myInstObj)
    elif myclassname == "CIM_UseOfLog":
        inst = CAssocUseOfLog(myclassname,myInstObj)
    elif myclassname == "CIM_LogManagesRecord":
        inst = CAssocLogManagesRecord(myclassname,myInstObj)
    elif myclassname == "CIM_AssociatedPrintInterpreter":
        inst = CAssocAssociatedPrintInterpreter(myclassname,myInstObj)
    elif myclassname == "CIM_AssociatedPrintSupply":
        inst = CAssocAssociatedPrintSupply(myclassname,myInstObj)
    else:
        TRC.trace(1,"error AssocClassFactory classname not found name|%s| id=|%s|"\
            %(myclassname))
    inst.xClassType = "association"
    return inst

# Specific association classes.
# Need to have separate methods to choose the instance keys properly. 

# C A s s o c P r i n t e r C o m p o n e n t 
# 
class CAssocPrinterComponent(CCimAssociation):
    def __init__(self,myClassname,myInstObj):
        TRC.trace(1,"enter CAssocPrinterComponent init name|%s| id=|%s|" \
            %(myClassname,id(self)))
        CCimAssociation.__init__(self,myClassname)
        self.sClassname = myClassname
        self.oInstObj = myInstObj
        CCimAssociation.__init__(self,self.sClassname)
        
    def isInstValid(self):
        return True

    def StoreKey(self):
        pass

# C A s s o c C o n c r e t e C o m p o n e n t 
# 
class CAssocConcreteComponent(CCimAssociation):
    def __init__(self,myClassname,myInstObj):
        TRC.trace(1,"enter CAssocConcreteComponent init name|%s| id=|%s|" \
            %(myClassname,id(self)))
        CCimAssociation.__init__(self,myClassname)
        self.sClassname = myClassname
        self.oInstObj = myInstObj
        CCimAssociation.__init__(self,self.sClassname)
        
    def isInstValid(self):
        return True

    def StoreKey(self):
        pass

# C A s s o c U s e O f L o g 
# 
class CAssocUseOfLog(CCimAssociation):
    def __init__(self,myClassname,myInstObj):
        TRC.trace(1,"enter CAssocUseOfLog init name|%s| id=|%s|" \
            %(myClassname,id(self)))
        CCimAssociation.__init__(self,myClassname)
        self.sClassname = myClassname
        self.oInstObj = myInstObj
        CCimAssociation.__init__(self,self.sClassname)

    def isInstValid(self):
        return True

    def StoreKey(self):
        pass

# C A s s o c L o g M a n a g e s R e c o r d 
# 
class CAssocLogManagesRecord(CCimAssociation):
    def __init__(self,myClassname,myInstObj):
        TRC.trace(1,"enter CAssocLogManagesRecord init name|%s| id=|%s|" \
            %(myClassname,id(self)))
        CCimAssociation.__init__(self,myClassname)
        self.sClassname = myClassname
        self.oInstObj = myInstObj
        CCimAssociation.__init__(self,self.sClassname)

    def isInstValid(self):
        return True

    def StoreKey(self):
        pass

# C A s s o c A s s o c i a t e d P r i n t I n t e r p r e t e r 
# 
class CAssocAssociatedPrintInterpreter(CCimAssociation):
    def __init__(self,myClassname,myInstObj):
        TRC.trace(1,"enter CAssocAssociatedPrintInterpreter init name|%s| id=|%s|" \
            %(myClassname,id(self)))
        CCimAssociation.__init__(self,myClassname)
        self.sClassname = myClassname
        self.oInstObj = myInstObj
        CCimAssociation.__init__(self,self.sClassname)

    def isInstValid(self):
        TRC.trace(1,"enter AssocInterp isInstValid class|%s|"%(self.sClassname))
        (self.lClasses,self.lKeys) = self.oInstObj.getInstInfo()
        TRC.trace(5,"proc  AssocInterp isInstValid class|%s| classes|%s| keys|%s|"\
            %(self.sClassname,self.lClasses,self.lKeys))
        self.sPrtNum = datastore.cPrinter.SnmpHrNumber
        self.sIpAddr = datastore.cPrinter.IpAddr
        self.sResult = 0
        # For each class and key pair, retrieve the channel's indices
        # that specify interpreters for the two functions.
        for self.idx in range(2):
            self.sRelatedClass = self.lClasses[self.idx]
            self.sRelatedKey = self.lKeys[self.idx]
            if self.sRelatedClass == "CIM_PrintChannel":
                # prtChannelCurrentJobCntlLangIndex
                self.sVarname = "prtChannelCurrentJobCntlLangIndex"
                self.sOid = fnsTransName2oid("CIM_PrintChannel",self.sVarname,self.sPrtNum,self.sRelatedKey)
                if self.sOid != None and self.sOid != "":
                    self.sResult = fnsGetSnmpValue(self.sClassname,self.sOid,self.sRelatedKey,self.sIpAddr)
                self.sChannelKey1 = self.sResult
                self.sChannelCurrentJobCntlLangIndex = self.sResult
                # prtChannelDefaultPageDescLangIndex
                self.sVarname = "prtChannelDefaultPageDescLangIndex"
                self.sOid = fnsTransName2oid("CIM_PrintChannel",self.sVarname,self.sPrtNum,self.sRelatedKey)
                if self.sOid != None and self.sOid != "":
                    self.sResult = fnsGetSnmpValue(self.sClassname,self.sOid,self.sRelatedKey,self.sIpAddr)
                self.sChannelKey2 = self.sResult
                self.sChannelDefaultPageDescLangIndex = self.sResult
                TRC.trace(3,"proc  AssocInterp IsInstValid key1|%s| key2|%s|"\
                    %(self.sChannelKey1,self.sChannelKey2))
            elif self.sRelatedClass == "CIM_PrintInterpreter":
                self.sInterpreterKey = self.sRelatedKey
                pass
            else:
                TRC.trace(1,"error AssocInterp isInstValid nomatch class|%s| classes|%s| keys|%s|"\
                    %(self.sClassname,self.lClasses,self.lKeys))
                return False
        # Compare here.  If either of the channel indices matches the
        # interpreter number, then keep this instance.
        if self.sChannelKey1 == self.sInterpreterKey:
            self.bValidity = True
            self.sType = "JobControlLanguage"
        elif self.sChannelKey2 == self.sInterpreterKey:
            self.bValidity = True
            self.sType = "PageDescriptionLanguage"
        else:
            self.bValidity = False
        TRC.trace(2,"exit  AssocInterp isInstValid class|%s| validity|%s|"\
            %(self.sClassname,self.bValidity))
        return self.bValidity

    def StoreKey(self):
        pass
        

# C A s s o c A s s o c i a t e d P r i n t S u p p l y 
# 
class CAssocAssociatedPrintSupply(CCimAssociation):
    def __init__(self,myClassname,myInstObj):
        TRC.trace(1,"enter CAssocAssociatedPrintSupply init name|%s| id=|%s|" \
            %(myClassname,id(self)))
        CCimAssociation.__init__(self,myClassname)
        self.sClassname = myClassname
        self.oInstObj = myInstObj
        CCimAssociation.__init__(self,self.sClassname)

    def isInstValid(self):
        TRC.trace(1,"enter AssocSupply isInstValid class|%s|"%(self.sClassname))
        (self.lClasses,self.lKeys) = self.oInstObj.getInstInfo()
        TRC.trace(5,"proc  AssocSupply isInstValid class|%s| classes|%s| keys|%s|"\
            %(self.sClassname,self.lClasses,self.lKeys))
        self.sPrtNum = datastore.cPrinter.SnmpHrNumber
        self.sIpAddr = datastore.cPrinter.IpAddr
        self.sResult = 0
        # For each class and key pair, retrieve the supply's index
        # that specifies the marker that uses it.
        for self.idx in range(2):
            self.sRelatedClass = self.lClasses[self.idx]
            self.sRelatedKey = self.lKeys[self.idx]
            if self.sRelatedClass == "CIM_PrintSupply":

                # prtMarkerSuppliesMarkerIndex
                self.sVarname = "prtMarkerSuppliesMarkerIndex"
                self.sOid = fnsTransName2oid("CIM_PrintSupply",self.sVarname,self.sPrtNum,self.sRelatedKey)
                if self.sOid != None and self.sOid != "":
                    self.sResult = fnsGetSnmpValue(self.sClassname,self.sOid,self.sRelatedKey,self.sIpAddr)
                self.sSupplyKey = self.sResult
                self.sSupplyMarkerIndex = self.sResult

                TRC.trace(3,"proc  AssocSupply IsInstValid supplykey|%s| supplymarkerindex|%s|"\
                    %(self.sRelatedKey,self.sSupplyMarkerIndex))

            elif self.sRelatedClass == "CIM_PrintMarker":
                self.sMarkerKey = self.sRelatedKey
                pass
            else:
                TRC.trace(1,"error AssocSupply isInstValid nomatch class|%s| classes|%s| keys|%s|"\
                    %(self.sClassname,self.lClasses,self.lKeys))
                return False
        # Compare here.  If the marker index matches the
        # supply marker index number, then keep this instance.
        if self.sSupplyKey == self.sMarkerKey:
            self.bValidity = True
        else:
            self.bValidity = False
        TRC.trace(2,"exit  AssocSupply isInstValid class|%s| classes|%s| keys|%s| validity|%s|"\
            %(self.sClassname,self.lClasses,self.lKeys,self.bValidity))
        return self.bValidity

    def StoreKey(self):
        pass

#=====================================

# f n s T r a n s N a m e 2 o i d ( )
# 
# Map SNMP variable name to usable numeric OID including appropriate subscripts.
def fnsTransName2oid(myclassname,myvarname,myprtnum,myrow):
    TRC.trace(1,"enter assoc fnsTransName2oid class|%s| var|%s| prt|%s| row|%s|"\
        %(myclassname,myvarname,myprtnum,myrow))
    # First get the base OID.
    sOid = dName2oid.mget(myvarname)
    # Get the SNMP subscripts needed for this OID.  
    # Check first for exceptions; if not, then lookup normally.  
    sSubs = dVar2subs.mget(myvarname)
    sTablename = dClass2table.mget(myclassname)
    if sSubs is None:
        sSubs = dTable2subs.mget(sTablename)
    TRC.trace(5,"proc  assoc translateName2oid oid|%s| class|%s| table|%s| subs|%s|" \
        % (sOid,myclassname,sTablename,sSubs))
    sFullOid = sOid
    # Paste proper subscripts onto partial OID.
    # Careful: these are not all mutually exclusive: prt and row 
    # occur together.  
    if sFullOid is not None:
        if sSubs.find("none") >= 0:
            sFullOid += ".0"
        if sSubs.find("scalar") >= 0:
            sFullOid += ".0"
        if sSubs.find("prt") >= 0:
            sFullOid += "." + myprtnum
        if sSubs.find("row") >= 0:
            sFullOid += "." + str(myrow)
        if sSubs.find("index") >= 0:
            sFullOid += "." + "index"
        if sFullOid == sOid:  
            sFullOid += "." + "err: bad subscript spec"
        # Now actually retrieve the value.  
    TRC.trace(1,"exit  assoc translateName2oid class|%s| var|%s| row|%s| oid|%s|"\
        %(myclassname,myvarname,myrow,sFullOid))
    return sFullOid

# g e t S n m p V a l u e ( )
# Finally, actually retrieve the SNMP variable value from the device.  
def fnsGetSnmpValue(myclassname,myoid,myrow,myipaddr):
    TRC.trace(1,"enter assoc getSnmpValue class|%s| oid|%s| ip|%s|"\
        %(myclassname,myoid,myipaddr))
    if myoid.find("index") >= 0:
        sResult = myrow
    else:
        sCmd = cSnmpCli.cliGet(myipaddr,myoid)
        sResult = cSnmpCli.cliDo(sCmd)
        #TEMP
        # remove this when we write code to parse snmpget output
        #valueVar = -1  # not here yet
        #TEMP
        # maybe, okay for mem version of cliDo that parses get command itself
    TRC.trace(2,"exit  CCimProp getsnmp class|%s| row|%s| oid|%s| ip|%s| val|%s|"\
        %(myclassname,myrow,myoid,myipaddr,sResult))
    return sResult

#end
