#!/usr/bin/python
#  calcsetup.py                     RBLandau 20080913
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

# These are the registration functions for the runtime calculation 
# functions that produce property values for CCimProp instances.  

# The functions in calcvalues.py are used to calculate the values of 
# CIM properties.  They will be called by name from the getValue
# methods of the CimProp instances.  They depend on the bin full
# of SNMP variables already having been retrieved.  

from NewTrace import TRC

from dicts      import *
from sys        import modules
from calcvalues import *

#=========== G e n e r a l  r e g i s t r a t i o n   r o u t i n e s

# f R e g i s t e r F u n c t i o n ( )
#
# Register the function name in the instance dictionary.
# Complains, only mildly, if a function name is duplicated, that is, 
# used twice.  Of course, some functions are used many times, such as
# fNull and fDirect and fEnums.  
def fRegisterFunction(mysFuncname):
    if dFunc2inst.has_key(mysFuncname):
        TRC.tracef(3,"CALC","proc  calc fRegisterFunction duplicate funcname|%s|"%mysFuncname)
    else:
        try:
            modules["calcvalues"]
        except (KeyError): 
            from calcvalues import *
        if fbCallable(modules["calcvalues"],mysFuncname):
            dFunc2inst.mset(mysFuncname,eval(mysFuncname))
            TRC.tracef(3,"CALC","exit  calc fRegisterFunction name|%s|"%mysFuncname)
        else:
            TRC.tracef(2,"CALC","error function not callable|%s|"%mysFuncname)

# f b C a l l a b l e ( )
#
# Check to see that the function we are trying to register is not a ringer.
# If there is no such function, complain so that someone writes it or 
# corrects the spelling of its name.  
def fbCallable(module_object, function_name_string): 
    "Check if an attribute is available in a given module and if it is callable" 
    if hasattr(module_object, function_name_string): 
        function_object = getattr(module_object, function_name_string) 
        if callable(function_object): 
            return True
        else:
            TRC.tracef(2,"CALC","error function %s:%s exists but is not callable" % (module_object.__name__, function_name_string))
            return False
    else:
        TRC.tracef(2,"CALC","error %s:%s does not exist" % (module_object.__name__, function_name_string))
        return False

# f C a l l B y N a m e ( )
#
# Function to call the mapped function for this property.  
# Called by the CCimProp instance to generate the value.
def fCallByName(myinstref,myfuncname,myvarname=""):
    if dFunc2inst.has_key(myfuncname):
        temp = dFunc2inst.mget(myfuncname)(myinstref,myvarname)
    else:
        temp = "NYI: function not written |%s|"%myfuncname
    return temp

# f R e g i s t e r A r g V a r ( )
#
# Register the name of an SNMP variable used to calculate
# some property.  Some mapping function will use
# the name found here.  
# Regardless of the function used, all the arguments should 
# be registered here.  Since the key is classname and propname, 
# there is no ambiguity about the end use of the argument.  
def fRegisterArgVar(mypropname,myvarname):
    dArgprop2var.mset(mypropname,myvarname)
    TRC.tracef(3,"CALC","exit  calc fRegisterArgVar prop|%s| var|%s|"\
        %(mypropname,myvarname))


# f R e g i s t e r S t a t u s V a r ( )
#
# Register the name of an SNMP var used to calculate 
#def fRegisterStatusVar(mypropname,myvarname):
#    dStatusprop2var.mset(mypropname,myvarname)
#    TRC.tracef(3,"CALC","exit  calc fRegisterStatusVar prop|%s| var|%s|"\
#        %(mypropname,myvarname))
# BZZZT, not used.  Kill.  

'''
Criteria for a property to get calculated properly.  

- The property belongs to a class in classprops.ini.
- The property has a bin of SNMP variables used for the calculation in bin.ini.
- The property has an algorithm that calculates the value in algs.ini.
- The property algorithm may be one of the canned functions available, 
    such as fDirect, fDeprecated, fNull, etc.
- The property algorithm name begins with lower case letter f.
- The property algorithm routine exists in calcvalues.py (or 
    elsewhere with appropriately imported names).  
- The algorithm routine stores the value in the propinst.value attribute.  
- There is another mechanism that was used at one point, but is no 
    longer active.  There is a key in the CimProp dictionary named 
    "value" that once held the answer, but is no longer used.  Store
    the value into the CimProp.value member attribute instead.
- Similarly, there was another mechanism at one point for returning
    list-valued values for array properties, but it is also no longer 
    used.  There was a CimProp dict key "valuelist" used to collect 
    the values of of the array into a list.  Then this was migrated to 
    the CimProp.valuelist member attribute, and then dropped entirely.  
- The class to which a property belongs need not be a real class.  It
    may be one of the synthetic classes, all named "ElFako_something"
    to be conspicuous.  



'''
