#!/usr/bin/python
# cimclass.py               RBLandau 20080920
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

from NewTrace import TRC
from string import join

from dicts      import *
from cimprop    import *

#==============================
# C C i m C l a s s
class CCimClass(object):
    '''
    An instance of some CIM class in the printer.  
    This is one instance of the parent subsystem
    '''
    '''
    Class CCimClassSubsystem
    - keeps list of instances
    - keys in CIM-XML string format? keypropname:keyvalue;...
    - dict CIM keys to instance ref, self.dKeys2Instref
    - dict CIM keys to SNMP table and row id, self.dKeys2Tablename, self.dKeys2Rowid
    - harder for associations, need to figure out
    - for SNMP table, need subscript info: just index or hrDeviceIndex too
    
    CimClassInstance
    - one instance for each instance of a subunit, e.g., each input tray
    - static SNMP table parent [no, all instances have same parent, factor up to CimClass]
    - static SNMP index structure [no, all instances have same index structure]
    - static CIM key (concatenation)
    '''
    def __init__(self,myClassname,mykey,myrowid=0):
        TRC.trace(1,"enter CCimClass init name|%s| key|%s| rowid |%s| id|%s|" \
        % (myClassname,mykey,myrowid,id(self)))
        self.sClassname = myClassname
        self.sKey = mykey
        self.sRow = myrowid
        self.dProp2inst = CDict("%s.%s.dProp2inst"%(self.sClassname,self.sKey))
        self.bDeleteMe = False
        TRC.trace(2,"exit  CCimClass init name|%s| key|%s| rowid |%s| id|%s|" \
        % (myClassname,mykey,myrowid,id(self)))

    # m a k e A l l P r o p s ( ) 
    # Instantiate all properties of this class, according to what was 
    # in the classprops.  
    def makeAllProps(self):
        TRC.trace(1,"enter CCimClass makeAllProps")
        self.lProps = dClass2props.mget(self.sClassname)
        # Instantiate all the properties of the class, 
        # if there are any, i.e., not parent abstract classes.
        if self.lProps != None:
            for self.sPropname in self.lProps:
                TRC.trace(5,"proc  CCimClass makeAllProps addprop|%s|" % (self.sPropname))
                self.makeOneProp(self.sPropname)
        TRC.trace(2,"exit  CCimClass makeAllProps")

    # m a k e O n e P r o p ( ) 
    # Instantiate one property of this class.
    def makeOneProp(self,mypropname):
        TRC.trace(1,"enter CCimClass makeOneProp propname|%s|"%(mypropname))
        self.cCimProp = CCimProp(mypropname,self.sRow,self,self.sClassname)
        self.dProp2inst.mset(mypropname,self.cCimProp)
        TRC.trace(2,"exit  CCimClass makeOneProp name|%s| id|%s|"%(mypropname,id(self.cCimProp)))
        return self.cCimProp

    # c l e a r A l l V a l u e s ( ) 
    # Clear values of all the properties of this class.  
    def clearAllValues(self):
        TRC.trace(1,"enter CCimClass clearValues class|%s| rowid|%s|"\
            %(self.sClassname,self.sRow))
        for self.sPropname in sorted( self.dProp2inst.keys() ):
            self.dProp2inst.mget(self.sPropname).clearValue()
        TRC.trace(2,"exit  CCimClass clearValues")

    # f n M a r k F o r D e l e t i o n 
    # 
    def fnMarkForDeletion(self):
        TRC.trace(1,"enter CimClass MarkForDeletion class|%s| key|%s| rowid|%s|"\
            %(self.sClassname,self.sKey,self.sRow))
        self.bDeleteMe = True

    # g e n e r a t e A l l V a l u e s ( ) 
    # Calculate values of all the properties of this class.  
    def generateAllValues(self):
        TRC.trace(1,"enter CCimClass generateValues class|%s| rowid|%s|"\
            %(self.sClassname,self.sRow))
        for self.sPropname in sorted( self.dProp2inst.keys() ):
            self.dProp2inst.mget(self.sPropname).generateValue()
        TRC.trace(2,"exit  CCimClass generateValues")
        #self.getKeyList()   # TEMP

    # g e t A l l V a r s ( ) 
    # 
    def getAllVars(self):
        TRC.trace(1,"enter CCimClass getAllVars class|%s| rowid|%s|"\
            %(self.sClassname,self.sRow))
        for self.sPropname in sorted( self.dProp2inst.keys() ):
            self.dProp2inst.mget(self.sPropname).getAllVars()
        TRC.trace(2,"exit  CCimClass getAllVars")

    # g e t A l l X M L ( ) 
    # 
    def getAllXml(self):
        TRC.trace(1,"enter CCimClass getAllXml")
        for self.sPropname in sorted( self.dProp2inst.keys() ):
            self.dProp2inst.mget(self.sPropname).generateXml()
        TRC.trace(2,"exit  CCimClass getAllXml")

    # g e t A l l F l a t ( ) 
    # Return a list of strings, one per property, name=value.
    def getAllFlat(self):
        TRC.trace(1,"enter CCimClass getAllFlat class|%s| rowid|%s| mrkdel|%s|"\
            %(self.sClassname,self.sRow,self.bDeleteMe))
        if not self.bDeleteMe:
            self.lFlat = list()
            for self.sPropname in sorted( self.dProp2inst.keys() ):
                self.temp = self.dProp2inst.mget(self.sPropname).generateFlat()
                self.lFlat.append(self.temp)
            TRC.trace(2,"exit  CCimClass getAllFlat")
            return self.lFlat
        else:
            return None

    # g e t K e y L i s t ( )
    #
    # Return a list containing the keys of this class.
    # Almost all the substantive classes have InstanceID as the key,
    # so return that most of the time.  Printer has a different set
    # of keys.  
    # Association classes are on their own: they can override this method.  
    def getKeyList(self):
        TRC.trace(1,"enter CCimProp getKeyList class|%s|" % (self.sClassname))
        self.lKeys = list()
        if self.sClassname == "CIM_Printer":
            self.addKeyToList("SystemCreationClassName",self.lKeys)
            self.addKeyToList("SystemName",self.lKeys)
            self.addKeyToList("CreationClassName",self.lKeys)
            self.addKeyToList("Name",self.lKeys)
        else:
            self.addKeyToList("InstanceID",self.lKeys)
        TRC.trace(2,"exit  CCimProp getKeyList class|%s| keys|%s|" \
            % (self.sClassname,str(self.lKeys)) )
        return self.lKeys

    # a d d K e y T o L i s t ( )
    # To add a key to the list, create the property instance, 
    # populate it with vars, then calculate the property value.  
    def addKeyToList(self,mykeyname,mykeylist):
        if not self.dProp2inst.mget(mykeyname):
            self.cCimProp = self.makeOneProp(mykeyname)
        self.cCimProp.getAllVars()
        self.value = self.cCimProp.generateValue()
        self.keystring = "%s=\"%s\""%(mykeyname,self.value)
        mykeylist.append(self.keystring)
        return mykeylist




#==============================
# C C i m A s s o c i a t i o n 
class CCimAssociation(CCimClass):
    '''
    Child class for CIM assocations.  
    Tricky.  Many more things have to be calculated on the fly.  
    Generally, an association contains the keys of two related class instances
    and maybe a few properties about the relationship.  
    '''
    def __init__(self,myClassname):
        TRC.trace(1,"enter CCimAssociation init name|%s| id|%s|" \
        % (myClassname,id(self)))
        CCimClass.__init__(self,myClassname,0,0)
        self.sClassname = myClassname
        pass

    # makeOneProp()
    # 
    def makeOneProp(self,mypropname):
        TRC.trace(1,"enter CCimAssociation makeOneProp propname|%s|"%(mypropname))
        self.sRow = 0   # Doesn't compute for associations.
        self.cCimProp = CCimProp(mypropname,self.sRow,self,self.sClassname)
        self.dProp2inst.mset(mypropname,self.cCimProp)
        TRC.trace(2,"exit  CCimClass makeOneProp name|%s| id|%s|"\
        %(mypropname,id(self.cCimProp)))
        return self.cCimProp

    # isInstValid()
    #
    # Used after instance is created to see if it should be allowed to live.  
    # 
    def isInstValid(self):
        return True


