#!/usr/bin/python
# cimprop.py
# RBLandau 20080808
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

from NewTrace   import TRC
from dicts      import *
from snmpcli    import cSnmpCli
from datastore  import *
from calcsetup  import *

#==============================
# C C i m P r o p 
class CCimProp(CDict):
    '''
    Class for a CIM property, to collect all info about the property.
    A property is a dictionary with entries for all the input SNMP vars and the output value.
    Propname is cimclassname dot propnameinclass.
    '''
    def __init__(self, myname,myrow,myowner,myownername=""):
        TRC.trace(1,"enter CCimProp init name|%s| class|%s| row|%s| id|%s| ownerid|%s|"\
            %(myname,myownername,myrow,id(self),id(myowner)))
        CDict.__init__(self,"CCimProp.%s.%s"%(myownername,myname))
        # Instance data that can be used by any class method or calc function.  
        self.sPropname = myname
        self.sRow = str(myrow)
        self.refOwnerinst = myowner
        self.sClassname = myownername
        self.sFullname = "%s.%s"%(self.sClassname,self.sPropname)
        self.lBin = dProp2bin.mget(self.sFullname)
        TRC.trace(5,"proc  CCimProp init bin|%s|"%(str(self.lBin)))
        self.lObjects = []
        self.value = -1
        TRC.trace(2,"exit  CCimProp init name|%s| id|%s|"%(myname,id(self)))

    # c l e a r V a l u e ( )
    # Empty the value(s) at the beginning of an evaluation.  
    def clearValue(self):
        TRC.trace(1,"enter CCimProp clearValue class|%s| prop|%s| row|%s|"\
            %(self.sClassname,self.sPropname,self.sRow))
        self.mset("value",None)
        self.value = None
        self.valuelist = list()
        self.mset("valuelist",list())
        #TRC.trace(2,"exit  CCimProp clearValue")
    
    # g e t A l l V a r s ( )
    # Retrieve all the variables listed in the property's bin.  
    #
    # Possible improvement for this routine: 
    # The single most frequent error has been failing to add a variable
    # to the bin.  So, when we get all the vars for a property, be sure
    # to retrieve also whatever var is listed as an argument for the calc
    # function, if any.  If it is a dup, already retrieved, don't do it
    # again.  
    #
    def getAllVars(self):
        TRC.trace(1,"enter CCimProp getAllVars class|%s| prop|%s| row|%s|"\
            %(self.sClassname,self.sPropname,self.sRow))
        lVars = self.lBin
        self.valueVar = None
        if lVars != None:
            for self.sVarname in sorted(lVars):
                TRC.trace(3,"loop  CCimProp getAllVars varname|%s|"%self.sVarname)
                # get each prop thru snmpget to set value
                self.valueVar = None
                self.valueVar = self.getOneVar(self.sVarname,self.sRow)
                TRC.trace(3,"proc  CCimProp getAllVars stored var|%s| val|%s|"\
                    %(self.sVarname,self.valueVar))
        # One last thing.  If the parent class is an association, 
        # add in the names and indices of the related classes.  
#        self.lTempKeys = dAssociations.keys()
#        if (self.sClassname in self.lTempKeys):
        if hasattr(self.refOwnerinst,"xClassType") \
        and (self.refOwnerinst.xClassType == "association"):
            self.mset("lRelatedClassname",self.refOwnerinst.lRelatedClassname)
            self.mset("lSNMPRowId",self.refOwnerinst.lSNMPRowId)
        TRC.trace(2,"exit  CCimProp getAllVars class|%s| prop|%s| val|%s|"\
            %(self.sClassname,self.sPropname,self.valueVar))

    # g e t O n e V a r ( )
    # Retrieve one variable, by name and row.  
    # This is done separately so that it can be called individually
    # to get vars for associations.
    def getOneVar(self,myvarname,myrow):
        TRC.trace(1,"enter CCimProp getOneVar  varname|%s|"%(myvarname))
        self.sPrtNum = datastore.cPrinter.SnmpHrNumber
        self.sOid = self.translateName2oid(myvarname,self.sPrtNum,myrow)
        if self.sOid != None and self.sOid != "":
            self.sFullOid = self.sOid
            self.sIpAddr = datastore.cPrinter.IpAddr
            self.sResult = self.getSnmpValue(self.sOid,self.sIpAddr)
            self.getvarValue = self.sResult
            self.mset(myvarname,self.getvarValue)
        else:
            #TEMP
            self.getvarValue = -2  # not implemented at all, no OID
            #TEMP
            self.mset(myvarname,self.getvarValue)
        TRC.trace(2,"exit  CCimProp getOneVar var|%s| returnval|%s|"\
            %(myvarname,self.getvarValue))
        return self.getvarValue

    # t r a n s l a t e N a m e 2 o i d ( )
    # Map SNMP variable name to usable numeric OID including appropriate subscripts.
    def translateName2oid(self,myvarname,myprtnum,myrow):
        TRC.trace(1,"enter CCimProp transname class|%s| prop|%s| var|%s| prt|%s| row|%s|"\
            %(self.sClassname,self.sPropname,myvarname,myprtnum,myrow))
        self.sOid = dName2oid.mget(myvarname)
        self.sClassprop = self.sPropname
        # Get the SNMP subscripts needed for this OID.  
        # Check first for exceptions; if not, then lookup normally.  
        self.sSubs = dVar2subs.mget(myvarname)
        self.sTablename = dClass2table.mget(self.sClassname)
        if self.sSubs == None:
            self.sSubs = dTable2subs.mget(self.sTablename)
        TRC.trace(5,"proc  CCimProp transname oid|%s| class|%s| table|%s| subs|%s|" \
            % (self.sOid,self.sClassname,self.sTablename,self.sSubs))
        self.sFullOid = self.sOid
        # Paste proper subscripts onto partial OID.
        # Careful: these are not all mutually exclusive: prt and row 
        # occur together.  
        if self.sFullOid != None:
            if self.sSubs.find("none") >= 0:
                self.sFullOid += ".0"
            if self.sSubs.find("scalar") >= 0:
                self.sFullOid += ".0"
            if self.sSubs.find("prt") >= 0:
                self.sFullOid += "." + myprtnum
            if self.sSubs.find("row") >= 0:
                self.sFullOid += "." + str(myrow)
            if self.sSubs.find("index") >= 0:
                self.sFullOid += "." + "index"
            if self.sFullOid == self.sOid:  
                self.sFullOid += "." + "err: bad subscript spec"
            # Now actually retrieve the value.  
        TRC.trace(1,"exit  CCimProp transname class|%s| prop|%s| var|%s| row|%s| oid|%s|"\
            %(self.sClassname,self.sPropname,myvarname,myrow,self.sFullOid))
        return self.sFullOid

    # g e t S n m p V a l u e ( )
    # Finally, actually retrieve the SNMP variable value from the device.  
    def getSnmpValue(self,myoid,myipaddr):
        TRC.trace(1,"enter CCimProp getsnmp class|%s| prop|%s| row|%s| oid|%s| ip|%s|"\
            %(self.sClassname,self.sPropname,self.sRow,myoid,myipaddr))
        if myoid.find("index") >= 0:
            self.sResult = self.sRow
        else:
            self.sCmd = cSnmpCli.cliGet(myipaddr,self.sFullOid)
            self.sResult = cSnmpCli.cliDo(self.sCmd)
            #TEMP
            # remove this when we write code to parse snmpget output
            #self.valueVar = -1  # not here yet
            #TEMP
            # maybe, okay for mem version of cliDo that parses get command itself
        TRC.trace(1,"exit  CCimProp getsnmp class|%s| prop|%s| row|%s| oid|%s| ip|%s| val|%s|"\
            %(self.sClassname,self.sPropname,self.sRow,myoid,myipaddr,self.sResult))
        return self.sResult

    # g e n e r a t e V a l u e ( )
    # Calculate the property value from the bin of variables.  
    # Function to calculate the value is called by the registered name,
    # if there is one, with the registered argument, if there is one.  
    def generateValue(self):
        TRC.trace(1,"enter CCimProp generateValue class|%s| prop|%s| row|%s|"\
            %(self.sClassname,self.sPropname,self.sRow))
        # Get the name of the mapping function assigned to this property.
        # If there isn't one, complain that there is no function declared.  
        self.sFuncname = dProp2alg.mget(self.sFullname)
        if self.sFuncname:
            self.varname = dArgprop2var.mget(self.sFullname)
            self.value = fCallByName(self,self.sFuncname,self.varname)
        else:
            self.value = "NYI: calc name not assigned %s"%self.sFullname
        TRC.trace(2,"exit  CCimProp generateValue prop|%s| row|%s| val|%s|"\
            %(self.sFullname,self.sRow,self.value))
        return self.value

    # g e t V a l u e ( )
    # Return the value, whatever type it may be.  
    def getValue(self):
        TRC.trace(1,"enter CCimProp getValue class|%s| prop|%s| row|%s|"\
            %(self.sClassname,self.sPropname,self.sRow))
        self.temp = self.mget("value")
        TRC.trace(2,"exit  CCimProp getValue")
        return self.temp

    # g e n e r a t e X m l ( )
    def generateXml(self):
        TRC.trace(1,"enter CCimProp genXml class|%s| prop|%s| row|%s|"\
            %(self.sClassname,self.sPropname,self.sRow))
        self.temp = "real XML goes here"
        TRC.trace(2,"exit  CCimProp genXml")
        return(self.temp)

    # g e n e r a t e F l a t ( )
    # Produce a single-line version suitable for debugging or QA.
    # Returns string:
    #   classname.propname = propvalue (no newline)
    # propvalue, if it is an enum, may have a text value appended to it.
    def generateFlat(self):
        TRC.trace(1,"enter CCimProp genFlat class|%s| prop|%s| row|%s|"\
            %(self.sClassname,self.sPropname,self.sRow))
        if self.value == "":
            self.value = "((empty string))"
        elif self.value == None:
            self.value = "((null))"
        # If the property is enumerated, there should be entries in the 
        # dPropenum2text dictionary mapping the numbers to the text.  
        # If there is one, append it to the numeric value so that the
        # reader can understand it easily.  
        self.sEnumKey = "%s=%s"%(self.sFullname,self.value)
        self.sEnumVal = dPropenum2text.mget(self.sEnumKey)
        TRC.trace(5,"proc  CimProp.generateFlat prop|%s| enumkey|%s| enumval|%s|"%\
            (self.sFullname,self.sEnumKey,self.sEnumVal))
        if self.sEnumVal:
            self.value = str(self.value) + " (%s)"%(self.sEnumVal)
        # Format the line neatly for easy reading.  
        self.temp = "%-45s = %s"%(self.sFullname,self.value)
        TRC.trace(2,"exit  CCimProp genFlat val|%s|"%(self.temp))
        return(self.temp)


