#!/usr/bin/python
# dicts.py
# RBLandau 20080808
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

from NewTrace import TRC

#==============================
# C D i c t 
class CDict(dict):
    '''
    General dictionary class, into which I will probably 
    slide logging to track use if we run into problems
    '''
    def __init__(self,myname="dictwithnoname"):
        dict.__init__(self)
        TRC.tracef(1,"CDIC","enter CDict init name|%s| id|%s|"%(myname,id(self)))
        self.Name = myname
    '''
    To get anything into or out of this class, or any child of this class, 
    use these functions.
    1.  If you use dictname[key], it will fault if the key is absent.
    2.  The mset and mget functions let us log their activities.
    '''
    def mset(self,mykey,myval):
        self[mykey] = myval
        TRC.tracef(5,"WDIC","proc  cdict %s mset|%s| value|%s|" % (self.Name,mykey,myval))
        return myval

    def mget(self,mykey):
        if self.has_key(mykey):
            self.temp = self.get(mykey)    # no exceptions for bad key
            TRC.tracef(5,"RDIC","proc  cdict %s mget|%s| value|%s|" % (self.Name,mykey,self.temp))
        else:
            TRC.tracef(2,"RDIC","proc  cdict %s badkey mget|%s|" % (self.Name,mykey))
            self.temp = None
        return self.temp

    def mrem(self,mykey):
        self.temp = self.get(mykey)
        del self[mykey]
        TRC.tracef(5,"WDIC","proc  cdict %s mrem|%s| value|%s|" % (self.Name,mykey,self.temp))
        return mykey

#==============================
# Dictionaries used here to hold various goodies

# name2oid: key = snmpobjectname, e.g., prtInputName
dName2oid = CDict("dName2oid")

# oid2name: key = oid with no leading dot
dOid2name = CDict("dOid2name")

# prop2bin: key = (cimclassname,cimpropertyname)
dProp2bin = CDict("dProp2bin")

# class2props: key = cimclassname, value = list of property names
dClass2props = CDict("dClass2props")

# class2table: key = classname, val = name of snmp table entry
dClass2table = CDict("dClass2table")

# prop2alg: key = (classname.cimpropname), val = name of mapping algorithm
dProp2alg = CDict("dProp2alg")

# table2subs: key = (tablename), val = subscript string from table.ini
dTable2subs = CDict("dTable2subs")

# var2subs: key = snmpvarname, val = subscript index indicators
#   prt, row, prt+row, index, or none
# This dictionary includes *exceptions* *only*.  If the vanilla algorithm 
# gets the right answer, don't put anything in here.  This exception 
# dictionary should be accessed first so that it overrides the normal algorithm.  
dVar2subs = CDict("dVar2subs")

# class2subs: key = (classname), val = subscript string from table.ini
dClass2subs = CDict("dClass2subs")

# class2keys: key = classname, val = list of SNMPRowId keys of instances
dClassname2instkeys = CDict("dClassname2instkeys")

# papersize2limits: key = papersizename, val = pair of (minsize,maxsize) in mm
dPapersize2limits = CDict("dPapersize2limits")

# propenum2text: key=classname.propname.valuemapnumber, val = valuename
dPropenum2text = CDict("dPropenum2text")

# Dictionaries and lists for association classes.

# assoc2groups: key = assocname.sequencenr, val = instance of CAssocGroup
dAssoc2groups = CDict("dAssoc2groups")
# Really cheap one just to get the unadorned name.
# assockey2name: key = assocname.sequencenr, val = assocname
dAssocKey2names = CDict("dAssocKey2names")
# One to answer question: is this classname an association?
# But does not uniquely identify the association use; be careful.
# dAssociations: associationclassname => same.
dAssociations = CDict("dAssociations")

# Set of indexed arrays to describe associations, related classes, and instances.  
# Just a list of association classnames in the keys.
lAssociations = list()
# List of classname pairs that are ends of the association.
lAssocClasses = list()
# List of instance (SNMP) keys of the two classes connected by an association.  
# Each entry is a list of the instance key pairs of the two classes.  
lAssocClassKeys = list()
# List of instances of CAssocGroup
# Each instance contains a list of instances of the association
#  between two particular classes.
lAssocGroups = list()


#=========== D i c t i o n a r i e s  f o r   r u n t i m e   c a l c u l a t i o n s 

# functionname -> function instance ref
dFunc2inst = CDict("dFunc2inst")

# For ALL mappings that take one argument, cimclassname.cimpropname -> snmpvarname
dArgprop2var = CDict("dArgprop2var")

