#!/usr/bin/python
#  printer.py                     RBLandau 20080821
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

from NewTrace import TRC

from dicts      import *
from subsystem  import *
from assoc      import *
from datastore  import *

#==============================
# C P r i n t e r
#
# Printer object.  One instance per known network printer.  
# For the moment, holds some static data for a printer; init values here
# to be overridden by the InitPrinters method in Provider. 
# Instantiate all the subsystems of the printer.   
class CPrinter(object):
    def __init__(self,myname="printerwithnoname"):
        TRC.trace(1,"enter CPrinter init name|%s| id|%s|"%(myname,id(self)))
        # Fill in recognizable default values for printer.  
        self.SnmpHrNumber = 1
        self.Name = myname
        self.IpAddr = "1.1.1.1"
        self.SnmpVersion = 1
        self.SnmpCommunityName = "public"
        # Store a key, sort of, for this one instance of Printer.  
        self.SystemCreationClassname = "CIM_ComputerSystem"
        self.SystemName = myname
        self.CreationClassname = "CIM_Printer"
        self.lPrinterKey = [ ["SystemCreationClassname",self.SystemCreationClassname], 
                            ["SystemName",self.SystemName], 
                            ["CreationClassname",self.CreationClassname], 
                            ["Name",self.Name] 
                           ]
        dClassname2instkeys.mset("CIM_Printer",self.lPrinterKey)
        # Dictionary of all subsystems of printer, by CIM classname.
        self.dClassname2subsysinst = CDict("myname.%s"%(self.SnmpHrNumber))
        # Create a subsystem container for each class.
        # Do all the substantive classes first, then all the associations.  
        TRC.trace(1,"proc  CPrinter init create subsystems for substantive classes")
        for self.sClassname in dClass2table.keys():
            TRC.trace(3,"loop  CPrinter subsys|%s|"%self.sClassname)
            lTempKeys = dAssociations.keys()
            TRC.trace(5,"proc  CPrinter assocnames|%s|"%lTempKeys)
            # Store ref to instance of subsystem.
            TRC.trace(5,"proc  CPrinter assoc is |%s| in |%s|? y/n|%s|" % \
                (self.sClassname,lTempKeys,(self.sClassname in lTempKeys)))
            if (self.sClassname in lTempKeys):
                #self.cSubsystem = CAssocSubsystem(self.sClassname)
                pass
            else:
                self.cSubsystem = CSubsystem(self.sClassname)
                self.dClassname2subsysinst.mset(self.sClassname,self.cSubsystem)
        TRC.trace(2,"exit  CPrinter init")

    # Now create all the association classes.
    # Associations cannot be instantiated until after all the substantive
    # class instances have been evaluated.  
    def initPhase2(self):
        TRC.trace(1,"enter CPrinter|%s| initPhase2 create subsystems for association classes"\
            %(self.Name))
        for self.sClassname in dClass2table.keys():
            TRC.trace(3,"loop  CPrinter subsys|%s|"%self.sClassname)
            lTempKeys = dAssociations.keys()
            TRC.trace(5,"proc  CPrinter assocnames|%s|"%lTempKeys)
            # Store ref to instance of subsystem.
            TRC.trace(5,"proc  CPrinter assoc is |%s| in |%s|? y/n|%s|" % \
                (self.sClassname,lTempKeys,(self.sClassname in lTempKeys)))

            # If the name is an association, construct it as one.
            # Otherwise it's a concrete subsystem.  
            if (self.sClassname in lTempKeys):
                self.cSubsystem = CAssocSubsystem(self.sClassname)
                self.dClassname2subsysinst.mset(self.sClassname,self.cSubsystem)
            else:
                #self.cSubsystem = CSubsystem(self.sClassname)  # assoc only in this phase
                pass
        TRC.trace(2,"exit  CPrinter initPhase2")
        return


    def getKeys(self,myclass):
        self.lKeys = dClassname2instkeys.mget(myclass)
        return self.lKeys


#==============================


