#!/bin/sh
# prov-test-analyze-log.sh
#                   Rick Landau, Dell, 20090330
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# Look at properties across the instances of printer
# to see whether the values seem reasonable and
# consistent.  

# t e s t 1 ( ) 
# Look for property name across all classes.
function test1()
{
    echo 
    echo '=============== Searching for' "$1"
    echo
    grep "$1" "$2"
}

# t e s t 2 ( ) 
# Look for property name within class.
function test2()
{
    echo 
    echo '=============== Searching for' "$1\\.$2"
    echo
    grep "$1\\.$2" "$3"
}

# t e s t c o m b o s ( ) 
# Look for list of property names across list of classes.
function testcombos()
{
echo -e "\n\n\n"'*********************************************************************************'
echo -e '********************' "Results ordered by Property, then Class" '********************'
echo -e '*********************************************************************************'"\n\n"
for prop in   $2
do
    for class in   $1
    do
        test2 "$class" "$prop" $3
    done
done
echo -e "\n\n\n"'*********************************************************************************'
echo -e '********************' "Results ordered by Class, then Property" '********************'
echo -e '*********************************************************************************'"\n\n"
for class in   $1
do
    for prop in   $2
    do
        test2 "$class" "$prop" $3
    done
done
}

if [ -z "$1" ] 
then
    echo Usage: $0 logfilename
    exit 1
fi

if [ ! -r "$1" ] 
then
    echo "Error: cannot find log file: $1"
    exit 2
fi
logfilename="$1"
echo Processing log file  "$logfilename"
echo

# M A I N   L I N E 

# First get all the important name and status properties.  
proplist="Name ElementName Description Caption InstanceID"
classlist="Printer AlertRecord Channel InputTray Interlock Interpreter Marker MediaPath OutputTray Supply"
testcombos "$classlist" "$proplist" "$logfilename"

proplist="PrimaryStatus OperatingStatus DetailedStatus CommunicationStatus AvailabilityStatus HealthState"
classlist="Printer AlertRecord Channel InputTray Interlock Interpreter Marker MediaPath OutputTray Supply"
testcombos "$classlist" "$proplist" "$logfilename"

# Get all capacity and unit properties for subsystems that have them.  
proplist="MaxCapacity MaxCapacityBasis CapacityUnit SupplyUnit RemainingCapacity RemainingCapacityBasis Type IsDefault"
classlist="InputTray OutputTray Supply"
testcombos "$classlist" "$proplist" "$logfilename"

# Get a few leftover status markers.  
proplist="CriticalAlertsPresent NonCriticalAlertsPresent"
classlist="Channel InputTray Interlock Interpreter Marker MediaPath OutputTray Supply"
testcombos "$classlist" "$proplist" "$logfilename"

# And clean up stuff that we didn't get otherwise.  

test1 "\\.CurrentNaturalLanguage" $1
test1 "\\.ConsoleNaturalLanguage" $1
test1 "\\.NaturalLanguagesSupported" $1
test1 "\\.CharSetsSupported" $1
test1 "\\.PaperTypesAvailable" $1
test1 "\\.DefaultPaperType" $1
test1 "\\.LanguagesSupported" $1
test1 "\\.MediaSizeName" $1
test1 "\\.MinMediaSizeName" $1
test1 "\\.MaxMediaSizeName" $1
test1 "\\.EnabledState" $1
test1 "\\.RequestedState" $1
test1 "\\.LifeCount" $1
test1 "\\.PowerOnCount" $1
test1 "\\.SNMPRowId" $1

test2 "Printer" "DeviceID" $1
test2 "Printer" "SerialNumber" $1
test2 "Printer" "SystemName" $1
test2 "Printer" "EnabledDefault" $1
test2 "Printer" "ConsoleDisabled" $1
test2 "Printer" "TransitioningToState" $1

test2 "AlertRecord" "Description" $1
test2 "AlertRecord" "LocalizedDescription" $1
test2 "AlertRecord" "ComponentClassName" $1
test2 "AlertRecord" "ComponentElementName" $1
test2 "AlertRecord" "RecordID" $1
test2 "AlertRecord" "RecordData" $1
test2 "AlertRecord" "CreationTimeStamp" $1
test2 "AlertRecord" "CodeName" $1
test2 "AlertRecord" "MessageID" $1
test2 "AlertRecord" "SeverityLevel" $1
test2 "AlertRecord" "Locale" $1

test2 "Channel" "ProtocolVersion" $1
test2 "Channel" "Type" $1

test2 "InputTray" "MediaName" $1
test2 "InputTray" "MediaColor" $1
test2 "InputTray" "MediaType" $1
test2 "InputTray" "MediaWeight" $1
test2 "InputTray" "MediaWeightBasis" $1

test2 "MediaPath" "MaxSpeed" $1
test2 "MediaPath" "Type" $1

test2 "Interpreter" "AddressabilityBasis" $1
test2 "Interpreter" "FeedAddressability" $1
test2 "Interpreter" "XFeedAddressability" $1
test2 "Interpreter" "LangType" $1
test2 "Interpreter" "LangLevel" $1
test2 "Interpreter" "DefaultCharSetIn" $1

test2 "Marker" "AddressabilityBasis" $1
test2 "Marker" "FeedAddressability" $1
test2 "Marker" "XFeedAddressability" $1
test2 "Marker" "ProcessColorants" $1
test2 "Marker" "SpotColorants" $1
test2 "Marker" "CounterUnit" $1
test2 "Marker" "LifeCount" $1
test2 "Marker" "PowerOnCount" $1
test2 "Marker" "MarkTech" $1
test2 "Marker" "IsDefault" $1

test2 "MediaPath" "MaxSpeedUnit" $1
test2 "MediaPath" "MaxSpeedBasis" $1
test2 "MediaPath" "MaxSpeed" $1
test2 "MediaPath" "MaxMediaSizeNameBasis" $1
test2 "MediaPath" "MaxMediaSizeName" $1
test2 "MediaPath" "MinMediaSizeNameBasis" $1
test2 "MediaPath" "MinMediaSizeName" $1
test2 "MediaPath" "LocalizedDescription" $1
test2 "MediaPath" "Type" $1
test2 "MediaPath" "IsDefault" $1

test2 "Interlock" "InterlockStatus" $1

test2 "Supply" "LocalizedDescription" $1
test2 "Supply" "Type" $1
test2 "Supply" "Classification" $1
test2 "Supply" "ColorantName" $1
test2 "Supply" "ColorantRole" $1
test2 "Supply" "ColorantTonality" $1
test2 "Supply" "SupplyUnit" $1
test2 "Supply" "MaxCapacityBasis" $1
test2 "Supply" "MaxCapacity" $1
test2 "Supply" "RemainingCapacityBasis" $1
test2 "Supply" "RemainingCapacity" $1

echo -e "\n\n\nEndEndEndEndEndEndEndEndEndEndEndEndEndEnd"

#end
