#!/usr/bin/python
# prov-test-enum-assoc.py
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

from provini    import *
from NewTrace   import *
from re         import *

def init():
    #==============================
    # Get input files

    cProv = CProvider()
    dIniFiles = dict()

    # Take in all named arguments of the form namestring=value, but only
    # ones on the approved name list.
    lArgNames = ["bins","oids","props","printers","tables","algs","varsubs","assoc","papersizes","propenums","cli"]
    regNamedArg = "^(\w+)=([-\w\.\/:_]+)$"
    for idx in range(1,len(argv)):
        sArg = argv[idx]
        mMatchObj = re.search(regNamedArg,sArg)
        if mMatchObj:
            sName = mMatchObj.group(1)
            sValue = mMatchObj.group(2)
            if sName in lArgNames:
                dIniFiles[sName] = sValue
                TRC.trace(1,"proc  found arg |%s|=|%s|"%(sName,sValue))
            else:
                TRC.trace(0,"error bad arg name |%s| in |%s|"%(sName,sArg))
        else:
            TRC.trace(0,"error bad arg format '%s'" % (sArg))

    cProv.InitAll(dIniFiles)
    
#==============================
# Simple tests for enumerations

def test1(mynum,myclassname):
    TRC.trace(0,"BEGIN TEST%s"%(mynum))
    cEnumRequest = CEnumerateRequest(myclassname)
    status = cEnumRequest.do()
    TRC.trace(0,"cEnumRequest.do() class|%s| returnstatus|%s|"\
        %(myclassname,status))
    ans = cEnumRequest.getAllFlat()
    print "==================================="
    print ans
    print ""
    print "==================================="
    TRC.trace(0,"END   TEST%s"%(mynum))
    print ""
    del cEnumRequest


#==============================
# Now actually test something

def main():

    TRC.trace(0,"BEGINBEGIN")
    init()
    TRC.trace(0,"END INIT")

    # Enumerate some association classes

    test1(201,"CIM_PrinterComponent")
    test1(202,"CIM_ConcreteComponent")
    test1(203,"CIM_UseOfLog")
    test1(204,"CIM_LogManagesRecord")
    test1(205,"CIM_Dependency")
    test1(206,"CIM_AssociatedPrintInterpreter")
    test1(207,"CIM_AssociatedPrintSupply")

    TRC.trace(0,"ENDEND")

    print "Done"


# Do all this testing only if this is really the main program.  

if __name__ == "__main__":
    main()

#end
