#!/usr/bin/python
#  request.py                     RBLandau 20080901
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# 

from sys import argv, exit
from NewTrace import TRC
import re
from string import join

from dicts      import *
from printer    import *
from cimclass   import *
from datastore  import *

#==============================

#==============================
# class  C G e t R e q u e s t 
#
# 
class CGetRequest(object):
    '''
    GetRequest class.
    Get all the properties of a single class instance.  
    '''
    def __init__(self,myclass):
        TRC.trace(1,"enter CGetRequest init class=|%s|" % myclass)
        self.sClassname = myclass
        TRC.trace(2,"exit  CGetRequest init class=|%s|" % myclass)

    # d o ( )
    # Get a single instance specified by the rowid key.      
    def do(self,mykey,myrowid):
        TRC.trace(1,"enter CGetRequest doGetOp class=|%s| key=|%s| row|%s|" \
            % (self.sClassname,mykey,myrowid))
        self.sRow = myrowid
        self.sKey = mykey
        if dClass2table.has_key(self.sClassname):
            self.cClass = CCimClass(self.sClassname,mykey,myrowid)
            self.cClass.makeAllProps()
            self.cClass.clearAllValues()
#            self.cClass.clearAllValues()
            self.cClass.getAllVars()
            self.cClass.generateAllValues()
            # more goes here, to generate the XML, for instance.
            pass
            self.sReturn = 0
        else:
            TRC.trace(1,"Error: CGetRequest unknown class: %s"%(self.sClassname))
            self.sReturn = -2
        TRC.trace(2,"exit  CGetRequest doGetOp class=|%s| key=|%s| row|%s|" \
            % (self.sClassname,mykey,myrowid))
        return self.sReturn

    def getXml(self,mykey,myrowid):
        pass

    # g e t F l a t ( )
    # Emit a flat version of all the class properties.          
    def getFlat(self,mykey,myrowid):
        TRC.trace(1,"enter GetRequest.getFlat key|%s| rowid|%s|"%(mykey,myrowid))
        self.lFlat = self.cClass.getAllFlat()
        self.sFlat = "\n".join(self.lFlat)
        return self.sFlat

    # g e t O n e F l a t ( )
    # Emit a flat version of all the class properties.  
    def getOneFlat(self):
        self.lFlat = self.cClass.getAllFlat()
        self.sFlat = "\n".join(self.lFlat)
        return self.sFlat




#==============================
# class  C E n u m e r a t e R e q u e s t 
#
# 
class CEnumerateRequest(object):
    '''
    EnumerateRequest class.
    '''
    def __init__(self,myclass):
        TRC.trace(1,"enter CEnumerateRequest init class|%s|" % myclass)
        self.sClassname = myclass
        TRC.trace(2,"exit  CEnumerateRequest init class|%s|" % myclass)
            
    def do(self):
        TRC.trace(1,"enter CEnumerateRequest doEnumerateOp class|%s|" % self.sClassname)

        # If the class is an association, that requires completely separate logic.
        if dAssociations.has_key(self.sClassname):
            self.tmpstatus = self.doAssoc()
            return self.tmpstatus

        # Create all instances of this class and evaluate them.
        self.lInstances = datastore.cPrinter.getKeys(self.sClassname)
        if self.lInstances == None:
            TRC.trace(2,"error CEnumerateRequest unknownclass|%s|"\
                %(self.sClassname))
            return 1    # Error: classname not found
        else:
            TRC.trace(3,"proc  CEnumerateAssocRequest doEnumerateAssocOp class|%s| keys|%s|" \
                % (self.sClassname,self.lInstances))
            '''
            Using the list of keys for this class.
            call Get once for each, and collect all the output.
            '''
            self.lAllGetInst = list()
            for self.sRow in self.lInstances:
                self.cGetRequest = CGetRequest(self.sClassname)
                self.lAllGetInst.append(self.cGetRequest)
                self.status = self.cGetRequest.do("SNMPRowId=%s"%self.sRow,self.sRow)
                TRC.trace(3,"loop  EnumRequest  row|%s| status|%s|"\
                    % (self.sRow,self.status))
            TRC.trace(2,"exit  CEnumerateRequest do class|%s|" \
                % (self.sClassname))
            return 0    # TEMP

    def getXml(self,mykey,myrowid):
        pass

    # g e t O n e F l a t ( )
    # Flatten the values of one Getop that we just did.  
    def getOneFlat(self,mygetreq):
        self.lFlat = mygetreq.getOneFlat()
        if self.lFlat is not None:
            self.sFlat = "\n\n====== %s key=%s rowid=%s ======\n\n" \
                % (self.sClassname,mygetreq.sKey,mygetreq.sRow)
            self.sFlat += "".join(self.lFlat)
            return self.sFlat
        else:
            return ""

    # g e t A l l F l a t ( )
    # Collect the flat output for all the Getop instances in the Enum.
    def getAllFlat(self):
        TRC.trace(1,"enter CEnumerateRequest getAllFlat class|%s|" % self.sClassname)
        self.sAllFlat = ""
        for self.cget in self.lAllGetInst:
            self.sOneFlat = self.getOneFlat(self.cget)
            self.sAllFlat += self.sOneFlat
        return self.sAllFlat
            
#==============================
# Enumerate for association classes, very different
#==============================

    def doAssoc(self):
        TRC.trace(1,"enter CEnumerateRequest doEnumerateAssocOp class|%s|" % self.sClassname)
        self.lInstances = datastore.cPrinter.getKeys(self.sClassname)
        # This can't possibly work, because a single association class may be involved
        # with several other classes.  Hmmm.  How do we do this?  
        if self.lInstances == None:
            TRC.trace(2,"error CEnumerateAssocRequest doAssoc unknownclass|%s|" \
                % (self.sClassname))
            return 1    # Error: classname not found
        else:
            TRC.trace(3,"proc  CEnumerateAssocRequest doEnumerateAssocOp class|%s| keys|%s|" \
                % (self.sClassname,self.lInstances))

            '''
            Find an AssocGroup in the list that matches the classname.
            Get its list of instances of that class.  
            For each instance, request the class instance, and give it the 
                instance object, which contains the SNMP keys of its related classes.
            '''
            self.lAllGetInst = list()
            self.sAllFlat = ""
            for self.oGroup in lAssocGroups:
                if self.oGroup.sAssocname == self.sClassname:
                    TRC.trace(3,"proc  CEnumerateAssocRequest got AssocGroup|%s| assoc|%s|"\
                        % (id(self.oGroup),self.oGroup.sAssocname))
                    self.lAssocInstances = self.oGroup.lInstances
                    for self.oClassInst in self.lAssocInstances:
                        self.cAssocGetRequest = CAssocGetRequest(self.sClassname,self.oClassInst)
                        self.lAllGetInst.append(self.cAssocGetRequest)
                        self.status = self.cAssocGetRequest.do()
                        TRC.trace(3,"loop  AssocEnumRequest status|%s|"\
                            % (self.status))
                        if self.status > 0:
                            del self.cAssocGetRequest
                else:
                    pass

            '''
            get the list of keys for this class.
            Call Get once for each, and collect all the output.
            '''
            '''
            self.lAllGetInst = list()
            self.sAllFlat = ""
            for self.sRow in self.lInstances:
                self.cGetRequest = CGetRequest(self.sClassname)
                self.lAllGetInst.append(self.cGetRequest)
                self.status = self.cGetRequest.do("SNMPRowId=%s"%self.sRow,self.sRow)
                TRC.trace(3,"loop  EnumRequest row|%s| status|%s|"\
                    % (self.sRow,self.status))
            TRC.trace(2,"exit  CEnumerateRequest do class|%s|" \
                % (self.sClassname))
            return 0    # TEMP
            '''


#==============================

# class  C A s s o c G e t R e q u e s t 

class CAssocGetRequest(object):
    def __init__(self,myClassname,myInstObj):
        '''
        string classname
        inst of CAssocInst
        '''
        TRC.trace(1,"enter CAssocGetRequest init class=|%s| classes|%s| rows|%s|"\
        % (myClassname,myInstObj.lRelatedClassname,myInstObj.lSNMPRowId))
        self.sAssocName = myClassname
        self.oInstObj = myInstObj
        # Now some fake properties to fool the real getOneFlat() function.
        self.sKey = myInstObj.lRelatedClassname
        self.sRow = myInstObj.lSNMPRowId
        #TRC.trace(2,"exit  CAssocGetRequest init class=|%s|" % (myClassname))
        
    def do(self):
        '''
        Build one association instance.  Add the parentage, related
        classes and row numbers, and evaluate all the properties.  
        If the instance was not a valid one, then just nuke it.  
        '''
        self.cAssocClass = AssocFactory(self.sAssocName,self.oInstObj)
        self.cAssocClass.lRelatedClassname = self.oInstObj.lRelatedClassname
        self.cAssocClass.lSNMPRowId = self.oInstObj.lSNMPRowId
        if self.cAssocClass.isInstValid():
            self.cAssocClass.makeAllProps()
            self.cAssocClass.clearAllValues()
            self.cAssocClass.getAllVars()
            self.cAssocClass.generateAllValues()
            status = 0  # success
            return status
        else:
            del self.cAssocClass
            status = 3  # fail: not a valid instance
        
    def getOneFlat(self):
        self.lFlat = list()
        self.lFlat.append("CAssocGetRequest list of lines for this instance goes here:\n")
        self.lFlat.append("    class|%s|related|%s|rows|%s|"\
            % (self.sAssocName,self.oInstObj.lRelatedClassname,self.oInstObj.lSNMPRowId))
        # Careful here: association instances get deleted if they 
        # are not valid key combinations.  
        if hasattr(self,"cAssocClass"):
            self.lFlat = self.cAssocClass.getAllFlat()
            if self.lFlat is not None:
                self.sFlat = "\n".join(self.lFlat)
            else:
                self.sFlat = None
        else:
            self.sFlat = None
        return self.sFlat

