#!/usr/bin/python
#  snmpcli.py                     RBLandau 20080822
# Interface to SNMP CLI utilities
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

from NewTrace   import TRC
from os         import popen
import re
from datastore  import *

class CSnmpCli(object):

    # g e t ( )
    # Format snmpget CLI command for property
    def cliGet(self,myIp,myOid):
        TRC.trace(1,"enter cliGet ip|%s| oid|%s|"%(myIp,myOid))
#        self.sLine = "sh snmpget.sh -v 1 -c public -O n %s %s" % (myIp,myOid)
        self.sLine = "perl snmpget.pl %s ini/snmpvalues.ini" % myOid
        TRC.trace(2,"exit  cliGet cmd|%s|"%(self.sLine))
        return self.sLine
    
    # g e t n e x t ( )
    # Format snmpgetnext CLI command for property
    def cliGetnext(self,myIp,myOid):
        TRC.trace(1,"enter cliGetnext ip|%s| oid|%s|"%(myIp,myOid))
#        self.sLine = "sh snmpgetnext.sh -v 1 -c public -O n %s %s" % (myIp,myOid)
        self.sLine = "perl snmpgetnext.pl %s ini/snmpvalues.ini" % myOid
        TRC.trace(2,"exit  cliGetnext cmd|%s|"%(self.sLine))
        return self.sLine
    
    # d o ( )
    # Issue CLI command and get stdout
    # Parse snmpget result
    def cliDo(self,myCommand):
        TRC.trace(1,"enter cliDo  cmd|%s|"%myCommand)
        self.sReturn = ""
        for self.sLine in popen(myCommand):
            self.sLine.rstrip()
            self.sReturn += self.sLine
        TRC.trace(2,"exit  cliDo result=|%s|"%self.sReturn)
        return self.sReturn

    # Super-special version that reads from memory.
    # Note that this just replaces the version right before it.  
    def cliDo(self,mycommand):
        TRC.trace(1,"enter cliDo (mem version) cmd|%s|"%mycommand)
        if mycommand.find("snmpgetnext.pl") >= 0:
            # g e t n e x t  command
            # Extract OID arg from command; transform to sortable form.
            self.regNextOid = "perl snmpgetnext.pl\s+([0-9\.]+)"
            self.mMatchobj = re.search(self.regNextOid,mycommand)
            if self.mMatchobj:
                self.sCommandOid = self.mMatchobj.group(1)
                # Search for first line with OID > arg. 
                self.regOidVal = "^([0-9\.]+)\t(.*)$"
                self.sSearchOid = fnsSortableOid(self.sCommandOid)
                self.fFound = 0
                for self.sLine in datastore.lCliData:
                    self.mMatchobj = re.search(self.regOidVal, self.sLine)
                    if self.mMatchobj:
                        self.sOidNum = self.mMatchobj.group(1)
                        self.sOidVal = self.mMatchobj.group(2)
                        self.sTempOid = fnsSortableOid(self.sOidNum)
                        if self.sTempOid > self.sSearchOid:
                            self.fFound = 1
                            break
                if self.fFound != 0:
                    self.sReturn = "%s\t%s"%(self.sOidNum,self.sOidVal)
                    TRC.trace(2,"exit  snmpgetnext mem found oid|%s| val|%s|" % (self.sOidNum,self.sOidVal))
                else:
                    self.sReturn = ""
                    TRC.trace(2,"exit  snmpgetnext mem OIDnotfound|%s|" % self.sCommandOid)
            else:
                self.Return = "Error: snmpgetnext command with no OID: %s"%mycommand
        elif mycommand.find("snmpget.pl") >= 0:
            # g e t  command
            # Extract OID arg from command; transform to sortable form.
            # Search for first line with OID = arg.  Empty string if not found.
            self.regNextOid = "perl snmpget.pl\s+([0-9\.]+)"
            self.mMatchobj = re.search(self.regNextOid,mycommand)
            if self.mMatchobj:
                self.sCommandOid = self.mMatchobj.group(1)
                # Search for first line with OID = arg. 
                self.regOidVal = "^([0-9\.]+)(.*)$"
                self.fFound = 0
                self.sReturn = "Error: OID not found %s"%self.sCommandOid
                for self.sLine in datastore.lCliData:
                    # Crack line into OID and value atoms.
                    self.mMatchobj = re.search(self.regOidVal, self.sLine)
                    if self.mMatchobj:
                        self.sOidNum = self.mMatchobj.group(1)
                        self.sOidVal = self.mMatchobj.group(2)
                        self.sOidOuterVal = self.sOidVal
                        # Strip leading white space and quotation marks from value, if present.
                        self.regNoQuote = "^(\t\"([^\"]*)\"|\t\'([^\']*)\'|\t(.*))$"
                        self.mMatchobj = re.search(self.regNoQuote, self.sOidOuterVal)
                        if self.mMatchobj:
                            self.sOidVal = self.mMatchobj.group(2)
                            if self.sOidVal == None:
                                self.sOidVal = self.mMatchobj.group(3)
                            if self.sOidVal == None:
                                self.sOidVal = self.mMatchobj.group(4)
                        else:
                            self.sOidVal = self.sOidOuterVal
                        TRC.trace(6,"proc  cliDo (mem) line oid|%s| val|%s|"%(self.sOidNum,self.sOidVal))
                        # If this OID matches the request, then done.
                        if self.sOidNum == self.sCommandOid:
                            self.sReturn = self.sOidVal
                            self.fFound = 1
                            TRC.trace(5,"proc  cliDo (mem) foundline|%s| oid|%s| val|%s|" % (self.sLine,self.sOidNum,self.sOidVal))
                            break
            else:
                self.sReturn = "Error: snmpget command with no OID: %s"%mycommand
        else:    
            self.Return = "NYI: cliDO mem version - unknown command"
            
        #
        TRC.trace(2,"exit  cliDo (mem) oid|%s| return|%s|" % (self.sCommandOid,self.sReturn))
        return self.sReturn

def fnsSortableOid(myoid):
    '''Return OID in string-sortable form with leading zeros.'''
    TRC.tracef(1,"OID","enter fnsSortableOid arg|%s|"%myoid)
    ltempNums = myoid.split(".")
    ltempNewNums = list()
    TRC.tracef(5,"OID","proc  fnsSortableOid list|%s|"%str(ltempNums))
    for stempNum in ltempNums:
        if stempNum != "":
            ltempNewNums.append("%06d" % int(stempNum))
    stempNewOid = ".".join(ltempNewNums)
    TRC.tracef(2,"OID","exit  fnsSortableOid out|%s|"%stempNewOid)
    return stempNewOid

# One instance we can all import and use.
cSnmpCli = CSnmpCli()

'''
Future plans for this module:

1.  Multimode access to SNMP information
- Entirely from MIB dump file, for development, debugging, and unit test. 
- Entirely from external access to the SNMP agent in the actual device, 
for the init phase of the provider.
- Cached access, partly from memory structures attached to the CPrinter
instance of the device, and partly live from the device.  This should be 
the normal mode when running real printers.  

2.  Ini files to guide caching.  
- mutable.ini tells us which SNMP variables change during the run 
so that their values must always be retrieved live on demand from 
the device.  

'''