#!/usr/bin/python
#  subsystem.py                     RBLandau 20080821
# 
# Copyright (c) 2008, 2009, Dell, Inc. 
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
#     * Redistributions of source code must retain the above
# copyright notice, this list of conditions and the following
# disclaimer.
# 
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided
# with the distribution.
# 
#     * Neither the name of the Dell, Inc., nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 

from NewTrace import TRC
from string import join
import re

from dicts      import *
from snmpcli    import cSnmpCli
from datastore  import datastore

#==============================
# C S u b s y s t e m
#
# Subsystem (subunit) class.  One instance per type of subunit.
#
class CSubsystem(object):
    '''
    Class for a printer subunit.
    For each class based on an SNMP table, collect the info 
    about instances:
    - keys of instance
    - SNMP id of the corresponding table row
    
    If the class is not based on an SNMP table, a separate class
    definition will be executed, instantiated, and the instref
    stored in the classname dictionary.  BFM.
    '''
    def __init__(self,myname=""):
        TRC.trace(1,"enter CSubsystem init name|%s| id=|%s|"%(myname,id(self)))
        self.Name = myname
        if myname != "":
            self.sEntryRoot = dClass2table.mget(myname)
            self.dClassname2instkeys = CDict("cSubsys.%s.dClassname2instkeys"%myname)
            self.lKeys = list()
            TRC.trace(3,"proc  CSubsystem init name|%s| tableroot|%s|"%(myname,self.sEntryRoot))
            if self.sEntryRoot != "none":
                # If the class has a root SNMP table...
                # Make (and fill) dictionary of instance ids and refs.
                #self.dInstkey2instref = CDict(myname+"."+"dInstkey2instref")
                #self.dInstrow2instref = CDict(myname+"."+"dInstrow2instref")
                # classname2inst: key = classname, val = list of instances keys of subsys
                self.sEntryOid = dName2oid.mget(self.sEntryRoot) + "."
                self.sNextOid = self.sEntryOid
                self.sPrevCol = ""  # First time thru, have not seen col id.
                TRC.trace(3,"proc  CSubsystem entry|%s| oid|%s|"%(self.sEntryRoot,self.sEntryOid))
                # Get the list of subscripts for this SNMP table.
                # Find an OID that matches the target, which is fooTable.
                # Save the last atom of the OID, which is the row number.  
                # GetNext until the column number changes.  
                while 1:
                    self.cmd = cSnmpCli.cliGetnext("9.9.9.9",self.sNextOid)
                    self.ans = cSnmpCli.cliDo(self.cmd)
                    # If the OID matches the search OID, save the 
                    # row number and get the column OID.
                    if self.ans.find(self.sEntryOid) >= 0:
                        self.sOid = self.ans.split()[0]
                        self.sNextOid = self.sOid
                        self.mMatchObj = re.search("^([\d\.]+)\.(\d+)$", self.sOid)
                        if self.mMatchObj:
                            self.sThisCol = self.mMatchObj.group(1)
                            self.sRow = self.mMatchObj.group(2)
                            # If still in the same column, or the first 
                            # time we found a match, save the row number. 
                            # Oh, and create the class instance for that row.  
                            TRC.trace(4,"proc  CSubsystem init row|%s| thiscol|%s| prevcol|%s|" % (self.sRow,self.sThisCol,self.sPrevCol))
                            if self.sPrevCol == self.sThisCol or self.sPrevCol == "":
                                # Save key and list of keys
                                self.sKey = "SNMPRowId=" + self.sRow
                                self.lKeys.append(self.sRow)
                                # Create an instance of the CIM class,
                                # which will eventually own properties
                                # per request.
                                #self.rowinst = CCimClass(myname,self.sKey,self.sRow)
                                # Save in dict by row number and full key.
                                #self.dInstrow2instref.mset(self.sRow,self.rowinst)
                                #self.dInstkey2instref.mset(self.sKey,self.rowinst)
                                self.sPrevCol = self.sThisCol
                            else:   # if not match column
                                break   # Done with this column.  
                    else:   # if not match the table
                        break
                # Store the list of instance keys for this subsystem,
                # for use in Enumerate.
                self.dClassname2instkeys.mset(myname,self.lKeys)
                dClassname2instkeys.mset(myname,self.lKeys)
                TRC.trace(4,"proc  CSubsystem init class|%s| listkeys|%s|" % (myname,str(self.lKeys)))
            else:   # If the subsystem has no root SNMP table...
                # Non-table class
                self.sKey = "singletoninstance=999"
                self.sRow = "999"
                self.lKeys = list()
                self.lKeys.append(self.sRow)
                self.dClassname2instkeys.mset(myname,self.lKeys)
                dClassname2instkeys.mset(myname,self.lKeys)
                #self.dInstkey2instref = CDict(myname+"."+"dInstkey2instref")
                #self.dInstrow2instref = CDict(myname+"."+"dInstrow2instref")
                #self.rowinst = CCimClass(myname,self.sKey,self.sRow)
                #self.dInstrow2instref.mset(self.sRow,self.rowinst)
                #self.dInstkey2instref.mset(self.sKey,self.rowinst)
                TRC.trace(2,"exit  CSubsystem init name|%s| table=none"%(myname))
        else:   # if class has no name, will be coded entirely by hand
            TRC.trace(2,"proc  CSubsystem init empty placeholder class with no name")
        TRC.trace(2,"exit  CSubsystem init name|%s|"%(myname))

    def getKeys(self):
        return self.lKeys


        